@@ -1,4 +1,4 @@
-/*      $Id: ir_remote.c,v 5.36 2008/03/30 14:53:06 lirc Exp $      */
+/*      $Id: ir_remote.c,v 5.38 2008/06/03 17:21:29 lirc Exp $      */
 
 /****************************************************************************
  ** ir_remote.c *************************************************************
@@ -37,6 +37,13 @@ struct ir_ncode *repeat_code;
 
 extern struct hardware hw;
 
+static int match_ir_code(struct ir_remote *remote, ir_code a, ir_code b)
+{
+	return ((remote->ignore_mask|a) == (remote->ignore_mask|b) ||
+		(remote->ignore_mask|a) == 
+		(remote->ignore_mask|(b^remote->toggle_bit_mask)));
+}
+
 void get_frequency_range(struct ir_remote *remotes,
 			 unsigned int *min_freq,unsigned int *max_freq)
 {
@@ -245,6 +252,13 @@ struct ir_ncode *get_code(struct ir_remote *remote,
 		post_mask = remote->toggle_bit_mask &
 		            gen_mask(remote->post_data_bits);
 	}
+	if(has_ignore_mask(remote))
+	{
+		pre_mask |= remote->ignore_mask >>
+			   (remote->bits + remote->post_data_bits);
+		post_mask |= remote->ignore_mask &
+		            gen_mask(remote->post_data_bits);
+	}
 	if(has_toggle_mask(remote) && remote->toggle_mask_state%2)
 	{
 		ir_code *affected,mask,mask_bit;
@@ -314,10 +328,9 @@ struct ir_ncode *get_code(struct ir_remote *remote,
 			ir_code next_all;
 
 			next_all = gen_ir_code(remote, remote->pre_data,
-								   get_ir_code(codes, codes->current),
-								   remote->post_data);
-			if(next_all==all ||
-			   next_all==(all^remote->toggle_bit_mask))
+					       get_ir_code(codes, codes->current),
+					       remote->post_data);
+			if(match_ir_code(remote, next_all, all))
 			{
 				found_code=1;
 				if(codes->next!=NULL)
@@ -376,10 +389,9 @@ struct ir_ncode *get_code(struct ir_remote *remote,
 						if(flag == 1)
 						{
 							next_all = gen_ir_code(remote, remote->pre_data,
-												   get_ir_code(codes, prev),
-												   remote->post_data);							
-							if(next_all==all ||
-							   next_all==(all^remote->toggle_bit_mask))
+									       get_ir_code(codes, prev),
+									       remote->post_data);
+							if(match_ir_code(remote, next_all, all))
 							{
 								codes->current = get_next_ir_code_node(codes, prev);
 								sequence_match = 1;
