@@ -1,4 +1,4 @@
-/*      $Id: lircd.c,v 5.76 2008/03/30 14:53:06 lirc Exp $      */
+/*      $Id: lircd.c,v 5.80 2008/10/04 21:48:43 lirc Exp $      */
 
 /****************************************************************************
  ** lircd.c *****************************************************************
@@ -406,7 +406,7 @@ void config(void)
 		logperror(LOG_ERR,NULL);
 		return;
 	}
-	config_remotes=read_config(fd);
+	config_remotes=read_config(fd, configfile);
 	fclose(fd);
 	if(config_remotes==(void *) -1)
 	{
@@ -515,10 +515,13 @@ void add_client(int sock)
 		{
 			if(!hw.init_func())
 			{
-				shutdown(clis[0],2);
-				close(clis[0]);
-				clin=0;
-				dosigterm(SIGTERM);
+				logprintf(LOG_WARNING,
+					  "Failed to initialize hardware");
+				/* Don't exit here, otherwise lirc
+				 * bails out, and lircd exits, making
+				 * it impossible to connect to when we
+				 * have a device actually plugged
+				 * in. */
 			}
 		}
 	}
@@ -876,6 +879,7 @@ void log_enable(int enabled)
 #ifdef USE_SYSLOG
 void logprintf(int prio,char *format_str, ...)
 {
+	int save_errno = errno;
 	va_list ap;
 	
 	if(!log_enabled) return;
@@ -883,6 +887,8 @@ void logprintf(int prio,char *format_str, ...)
 	va_start(ap,format_str);
 	vsyslog(prio, format_str, ap);
 	va_end(ap);
+
+	errno = save_errno;
 }
 
 void logperror(int prio,const char *s)
@@ -895,6 +901,7 @@ void logperror(int prio,const char *s)
 #else
 void logprintf(int prio,char *format_str, ...)
 {
+	int save_errno = errno;
 	va_list ap;  
 	
 	if(!log_enabled) return;
@@ -923,6 +930,7 @@ void logprintf(int prio,char *format_str, ...)
 		fputc('\n',stderr);fflush(stderr);
 		va_end(ap);
 	}
+	errno = save_errno;
 }
 
 void logperror(int prio,const char *s)
@@ -1999,7 +2007,7 @@ void loop()
 {
 	char *message;
 	
-	logprintf(LOG_NOTICE,"lircd(%s) ready",LIRC_DRIVER);
+	logprintf(LOG_NOTICE,"lircd(%s) ready", hw.name);
 	while(1)
 	{
 		(void) waitfordata(0);
