@@ -1,4 +1,4 @@
-/*      $Id: release.c,v 1.1 2007/05/06 11:54:02 lirc Exp $      */
+/*      $Id: release.c,v 1.3 2008/02/06 13:43:07 lirc Exp $      */
 
 /****************************************************************************
  ** release.c ***************************************************************
@@ -63,7 +63,7 @@ void register_button_press(struct ir_remote *remote, struct ir_ncode *ncode,
 	release_remote = remote;
 	release_ncode = ncode;
 	release_code = code;
-	release_gap = remote->remaining_gap;
+	release_gap = remote->max_remaining_gap;
 	
 	register_input();
 }
@@ -143,7 +143,7 @@ const char *release_map_remotes(struct ir_remote *old, struct ir_remote *new)
 	if(release_remote && is_in_remotes(old, release_remote))
 	{
 		if((remote = get_ir_remote(new, release_remote->name)) &&
-		   (ncode = get_ir_code(remote, release_ncode->name)))
+		   (ncode = get_code_by_name(remote, release_ncode->name)))
 		{
 			release_remote = remote;
 			release_ncode = ncode;
