@@ -16,7 +16,7 @@
  *   Vassilis Virvilis <vasvir@iit.demokritos.gr> 2006
  *      reworked the patch for lirc submission
  *
- * $Id: lirc_atiusb.c,v 1.69 2008/04/28 06:47:29 lirc Exp $
+ * $Id: lirc_atiusb.c,v 1.71 2008/05/19 08:10:35 uzuul Exp $
  */
 
 /*
@@ -67,7 +67,7 @@
 #include "drivers/kcompat.h"
 #include "drivers/lirc_dev/lirc_dev.h"
 
-#define DRIVER_VERSION		"$Revision: 1.69 $"
+#define DRIVER_VERSION		"$Revision: 1.71 $"
 #define DRIVER_AUTHOR		"Paul Miller <pmiller9@users.sourceforge.net>"
 #define DRIVER_DESC		"USB remote driver for LIRC"
 #define DRIVER_NAME		"lirc_atiusb"
@@ -180,8 +180,8 @@ static struct usb_device_id usb_remote_table [] = {
 	/* Microsoft Xbox DVD Movie Playback Kit IR */
 	{ USB_DEVICE(VENDOR_MS2, 0x0284) },
 
-	/* Some chinese manufacterer -- conflicts with the joystick from the
-	 * same manufacterer */
+	/* Some chinese manufacturer -- conflicts with the joystick from the
+	 * same manufacturer */
 	{ USB_DEVICE(VENDOR_MS3, 0xFFFF) },
 
 	/* Terminating entry */
@@ -1357,7 +1357,7 @@ static int __init usb_remote_init(void)
 	       DRIVER_VERSION "\n");
 	printk(DRIVER_NAME ": " DRIVER_AUTHOR "\n");
 	dprintk(DRIVER_NAME ": debug mode enabled: "
-		"$Id: lirc_atiusb.c,v 1.69 2008/04/28 06:47:29 lirc Exp $\n");
+		"$Id: lirc_atiusb.c,v 1.71 2008/05/19 08:10:35 uzuul Exp $\n");
 
 	request_module("lirc_dev");
 
